% The CarSim starting file
% Created and modified by Pierre Nugues
% Modified 27/11/2001

:- [utilities].
:- [parser]. % the partial parser
:- [tokenize]. % the tokenizer -- word breaker
:- [ontology]. % a small ontology of the domain
:- [statics]. % extracts the static objects
:- [collision_finder]. % finds the collisions
:- [info_extraction]. % the rest of the files
:- [init_directions]. % finds the initial directions of the vehicles
:- [fiche]. % writes the fiche describing the accident

%  FAIRE
% Il faut introduire une nouvelle catgorie d'objets mobiles en stationnement.
% Travailler le A13 et voir pourquoi definir avec defini/1 voiture 
% entrane autant de fautes 

:- 
	nl, nl, write('\tTo process all the reports, type:'), nl,
	write('\t?- process_reports.'), nl, nl,
	write('\tTo process one report, type:'), nl,
	write('\t?- carsim(+ReportID, -Report).'), nl,
	write('\tas in'), nl,
	write('\t?- carsim([''A1''], -Report).'), nl, nl.

 
%carsim(+ReportID, -Fiche)
% Input should be carsim(['A1'], L)
% or carsim(['B1', -, bis], L)
carsim(ReportID, [ReportID | Fiche]) :-
	read_corpus('corpus.xml', XMLCorpusText),
	parse_xml_corpus(CorpusText, XMLCorpusText, []),
	member([ReportID | ReportText], CorpusText),
	extract_information(ReportText, StaticObjects, DynamicObjects, Accidents),
	append(ReportID, ['.', ad], FileName),
	concat_atom(FileName, File),
	concat_atom(ReportID, ReportIDAtom),
	tell(File),
	write('// '), write('Report '),
	write(ReportIDAtom), nl,
	write_statics(StaticObjects),
	write_dynamics(DynamicObjects),
	write_accidents(Accidents), nl,
	told,
	append(StaticObjects, DynamicObjects, Objects),
	append(Objects, Accidents, Fiche),
	!.

/*
parse_carsim(ReportID, Fiche) :-
	read_corpus('corpus.xml', XMLCorpusText),
	parse_xml_corpus(CorpusText, XMLCorpusText, []),
	(   atom(ReportID),
	    member([[ReportID] | ReportText], CorpusText)
	;   %  for the reports with a bis suffix
	    member([ReportID | ReportText], CorpusText)
	),
	parse(ReportText, FirstSentence, ParsedSentence),
	write(ParsedSentence), nl.
*/
parse_carsim(ReportID, ParsedText) :-
	read_corpus('corpus.xml', XMLCorpusText),
	parse_xml_corpus(CorpusText, XMLCorpusText, []),
	(   atom(ReportID),
	    member([[ReportID] | ReportText], CorpusText)
	;   %  for the reports with a bis suffix
	    member([ReportID | ReportText], CorpusText)
	),
	parse_all_text(ReportText, ParsedText).


nouns_carsim(ReportID, ParsedText) :-
	read_corpus('corpus.xml', XMLCorpusText),
	parse_xml_corpus(CorpusText, XMLCorpusText, []),
	(   atom(ReportID),
	    member([[ReportID] | ReportText], CorpusText)
	;   %  for the reports with a bis suffix
	    member([ReportID | ReportText], CorpusText)
	),
	find_all_nouns(ReportText, ParsedText).



assert_list([H | T]) :-
	asserta(H),
	assert_list(T).
assert_list([]).

check_statics(L) :-
	read_corpus('corpus.xml', XMLCorpusText),
	parse_xml_corpus(CorpusText, XMLCorpusText, []),
	bagof([ReportID | StaticObjects],
	      ReportText^(member([ReportID | ReportText], CorpusText),
	      find_statics(ReportText, StaticObjects)), L).

write_reports_in_files :-
	read_corpus('corpus.xml', XMLCorpusText),
	parse_xml_corpus(CorpusText, XMLCorpusText, []),
	write_files(CorpusText).

write_files([[ID | ReportText] | RestReports]) :-
	(   ID = [ReportID]
	;
	   concat_atom(ID, ReportID) 
	),
	tell(ReportID),
	write('Texte '), write(ReportID), nl,
	write_text(ReportText),
	told,
	write_files(RestReports).
write_files([]).

write_text([Word | ReportText]) :-
	write(Word),
	write(' '),
	write_text(ReportText).
write_text([]) :- nl.

process_reports :-
	get_reportIDs(ID),
	process_reports(ID).

get_reportIDs(IDs) :-
	read_corpus('corpus.xml', XMLCorpusText),
	parse_xml_corpus(CorpusText, XMLCorpusText, []),
        bagof(ID, Report^member([ID | Report], CorpusText), IDs),
	!.

process_reports([]).
process_reports([ID | IDs]) :-
	carsim(ID, _),
	!,
	process_reports(IDs).

make_runtime :-
	qsave_program(carsim, [goal(process_reports), stand_alone(true)]).

% FONCTIONS D'INTERFACE

a(X):- acteur(X),nl,write(X).
d(X,Y):- init_direction(X,Y).


% affiche acteurs
aa :-
	findall(X, acteur(X),L),
	nl, write('voici les acteurs : '),nl,
	affiche_acteur(L).

affiche_acteur([]).
affiche_acteur([X|Q]) :-
	write(X), nl,
	affiche_acteur(Q).

% affiche les vehicules
av :-
	findall([X,Y,Z,T,U], vehicle(X,Y,Z,T,U), L),
	nl, write('voici les vehicules : '), nl,
	affiche_vehicule(L).

affiche_vehicule([]).
affiche_vehicule([X|Q]) :-
	write(X), nl,
	affiche_vehicule(Q).

% affiche les collisions
ao :-
	findall([X,Y,Z,T,U,V], collision2(X,Y,Z,T,U,V), L),
	nl, write('voici les collisions : '), nl,
	affiche_collision(L).

affiche_collision([]).
affiche_collision([X|Q]) :-
	write(X), nl,
	affiche_collision(Q).
% affiche les chocs
ac :-
	findall([X,Y,Z,T], choc(X,Y,Z,T), L),
	nl, write('voici les chocs : '), nl,
	affiche_chocs(L).

affiche_chocs([]).
affiche_chocs([X|Q]) :-
	write(X), nl,
	affiche_chocs(Q).


% affiche directions initiales
ad :-
	findall([X,Y], init_direction(X,Y), L),
	nl, write('voici les direction initiales : '), nl,
	affiche_init_directions(L).

affiche_init_directions([]).
affiche_init_directions([X| Q]) :-
	write(X), nl,
	affiche_init_directions(Q).


at(ReportText) :-
	parse_all_text(ReportText, T),
	write(T).

extract_information(ReportText, FinalStaticObjects, EnunciatorInfoActors, Accidents) :-
	find_statics(ReportText, StaticObjects),
	cree_acteurs(ReportText, InitialInfoActors, StaticObjects, NonSorted), 
	parenthesized_actors(ReportText, EntreParenthesesActors), 
	!,
	affine_acteurs(NonSorted, InitialInfoActors, EntreParenthesesActors, RefinedInfoActors),
	cree_directions_initiales(ReportText, RefinedInfoActors, DirectionInfoActors),
	reconcile_init_directions(StaticObjects, DirectionInfoActors, NewDirectionInfoActors),
	trouve_events(ReportText, NewDirectionInfoActors, EventInfoActors),
	cree_vehicules(EventInfoActors, VehicleInfoActors),
	default_events(StaticObjects, VehicleInfoActors, RefinedVehicleInfoActors),
	find_collisions(ReportText, StaticObjects, FinalStaticObjects, NonSorted, RefinedVehicleInfoActors, UpdatedInfoActors, Collisions),
	affine_enonciateur(Collisions, UpdatedInfoActors, EnunciatorInfoActors),
	%defaut_collision,
	affine_collision(EnunciatorInfoActors, Collisions, Accidents),
	write(EnunciatorInfoActors), nl,
	!,
/*	aa, % affiche acteurs
	ad, % affiche directions initiales
	av, % affiche les vehicules
	ao, % affiche les collisions
*/	nl.












