% Collision_finder.pl finds the collisions and the actors
% trouve les chocs et les acteurs avec une DCG

% trie tous les groupes de mots du texte et les range dans les listes ci dessous
% mots (ListeTotale, ListeActeurs, ListeChocs, ListeEvents).
mots([acteur(First) | Mots], [First | ListeActeurs], [acteur(First) | ListeChoc], [acteur(First) | ListeEvents]) -->
	mot(acteur(First)),
	{!},
	mots(Mots,ListeActeurs,ListeChoc,ListeEvents).
mots([direction(First) | Mots],ListeActeurs,ListeChoc,[direction(First)|ListeEvents]) --> mot(direction(First)), {!}, mots(Mots,ListeActeurs,ListeChoc,ListeEvents).
mots([verbe_mouvement(First) | Mots],ListeActeurs,ListeChoc,[verbe_mouvement(First)|ListeEvents]) --> mot(verbe_mouvement(First)), {!}, mots(Mots,ListeActeurs,ListeChoc,ListeEvents).
mots([statique(A) | Mots],[A|ListeActeurs],[statique(A)|ListeChoc],ListeEvents) --> mot(statique(A)), {!}, mots(Mots,ListeActeurs,ListeChoc,ListeEvents).
mots([verbe_choc(V) | Mots],ListeActeurs,[verbe_choc(V)|ListeChoc],ListeEvents) --> mot(verbe_choc(V)), {!}, mots(Mots,ListeActeurs,ListeChoc,ListeEvents).
mots([verbe_double_choc(V) | Mots],ListeActeurs,[verbe_double_choc(V)|ListeChoc],ListeEvents) --> mot(verbe_double_choc(V)), {!}, mots(Mots,ListeActeurs,ListeChoc,ListeEvents).
mots([First | Mots],ListeActeurs,ListeChoc,ListeEvents) --> mot(First), {!}, mots(Mots,ListeActeurs,ListeChoc,ListeEvents).
mots([],[],[],[]) --> [].


mot(acteur(A)) --> [ng(A)], {is_actor(A)}.
mot(acteur(A)) --> [pg(A)], {is_actor(A)}.
mot(statique(A)) --> [ng(A)], {is_static(A)}.
mot(statique(A)) --> [pg(A)], {is_static(A)}.
mot(PG)--> [pg(PrepGroup)], {type_dir(PrepGroup,PG)}.   
mot(PG)--> [rg(PrepGroup)], {type_dir(PrepGroup,PG)}.
mot(V) --> [vg(VerbeGroup)], {type_verbe(VerbeGroup,V)}.
mot(V) --> [ng(VerbeGroup)], {type_verbe(VerbeGroup,V)}.
mot(V) --> [gg(VerbeGroup)], {type_verbe(VerbeGroup,V)}.
mot(V) --> [infg(VerbeGroup)], {type_verbe(VerbeGroup,V)}.

mot(A) --> [A].

type_dir(PrepGroup,PG) :-
	dir(PG2,Type,_),
	subset(PG2,PrepGroup),
	PG=..[Type,PrepGroup].


type_verbe(VerbeGroup, V) :-
	verbe(Verbe, Type, _),
	subset(Verbe, VerbeGroup),
	V =.. [Type, VerbeGroup].

dir([gauche],direction,gauche).
dir([_gauche],direction,gauche).
dir([droite],direction,droite).
dir([_droite],direction,droite).
dir(['D'],direction,droite).

verbe([recevoir],verbe_choc,_).
verbe([faire,rentrer],verbe_choc,_).
verbe([N,viter],verbe_choc,_) :- member(N,[ne,n]).
verbe([heurter],verbe_choc,_).
verbe([casser],verbe_choc,_).
verbe([percuter],verbe_choc,_).
verbe([taper],verbe_choc,_).
verbe([toucher],verbe_choc,_).
verbe([abmer],verbe_choc,_).
verbe([accrocher],verbe_choc,_).
verbe([cogner],verbe_choc,_).
verbe([emboutir],verbe_choc,_).
verbe([accidenter],verbe_choc,_).
verbe([enfoncer],verbe_choc,_).
verbe([frotter],verbe_double_choc,_).
verbe([pousser],verbe_double_choc,_).
verbe([projeter],verbe_double_choc,_).

% verbe avec ngation
verbe([N,dmarrer],verbe_mouvement,verbe_stop):- member(N,[ne,n]).
verbe([N,arrter],verbe_mouvement,verbe_avancer):- member(N,[ne,n]).
verbe([N,freiner],verbe_mouvement,verbe_avancer):- member(N,[ne,n]).

verbe([arrter],verbe_mouvement,verbe_stop).
verbe([stopper],verbe_mouvement,verbe_stop).
verbe([stationner],verbe_mouvement,verbe_stop).
verbe([immobiliser],verbe_mouvement,verbe_stop).
verbe([circuler],verbe_mouvement,verbe_avancer).
verbe([rouler],verbe_mouvement,verbe_avancer).
verbe([dmarrer],verbe_mouvement,verbe_avancer).
verbe([redmarrer],verbe_mouvement,verbe_avancer).
verbe([avancer],verbe_mouvement,verbe_avancer).
verbe([tourner],verbe_mouvement,verbe_tourner).
verbe([virer],verbe_mouvement,verbe_tourner).

% cherche les choc dans la liste de mots ListeChoc
% sort les chocs en liste 
% la numrotation ne fonctionne pas
trouve_choc([],0) --> [].
trouve_choc([choc(A,B,V,N), choc(B,C,V,N2)|Reste],N2) -->
	sujet_choc(A),
	vdpronominalchoc(V,B),
	cible_choc(C),
	trouve_choc(Reste,M),
	{N is M + 1, N2 is N + 1}.
trouve_choc([choc(B,A,V,N), choc(A,C,V,N2)|Reste],N2) -->
	sujet_choc(A),
	vdchoc(V),
	cible_choc(C),
	agent_choc(B),
	trouve_choc(Reste,M),
	{N is M + 1, N2 is N + 1}.
trouve_choc([choc(B,A,V,N), choc(A,C,V,N2)|Reste],N2) -->
	agent_choc(B),
	sujet_choc(A),
	vdchoc(V),
	cible_choc(C),
	trouve_choc(Reste,M),
	{N is M + 1, N2 is N + 1}.
trouve_choc([choc(B,A,V,N), choc(A,C,V,N2)|Reste],N2) -->
	sujet_choc(A),
	vdchoc(V),
	agent_choc(B),
	cible_choc(C),
	trouve_choc(Reste,M),
	{N is M + 1, N2 is N + 1}.
trouve_choc([choc(A,B,V,N), choc(B,C,V,N2)|Reste],N2) -->
	sujet_choc(A),
	vdchoc(V), objet_choc(B),
	cible_choc(C),
	trouve_choc(Reste,M),
	{N is M + 1, N2 is N + 1}.
trouve_choc([choc(A,C,V,N)|Reste],N) -->
	sujet_choc(A),
	vdchoc(V),
	cible_choc(C),
	trouve_choc(Reste,M),
	{N is M + 1}.
trouve_choc([choc(B,A,V,N)|Reste],N) -->
	sujet_choc(A),
	vchoc(V),
	agent_choc(B),
	trouve_choc(Reste,M),
	{N is M + 1}.
trouve_choc([choc(A,B,V,N)|Reste],N) -->
	sujet_choc(A),
	vchoc(V),
	objet_choc(B),
	trouve_choc(Reste,M),
	{N is M + 1}.
trouve_choc([choc(A,B,V,N)|Reste],N) -->
	sujet_choc(A),
	vpronominalchoc(V,B),
	trouve_choc(Reste,M),
	{N is M + 1}.
trouve_choc(Reste,N) -->
	mot(_),
	trouve_choc(Reste,N).

sujet_choc(X) --> [acteur(X)].						% un acteur peut tre le sujet d'un choc
objet_choc(X) --> [acteur(X)].						% un acteur peut tre l'objet d'un choc
objet_choc(X) --> [statique(X)].					% un objet statique peut tre l'objet d'un choc
cible_choc([Prep|X]) --> [acteur([Prep|X])], {prep_sur(Prep)}.		% un acteur peut tre la cible d'un choc
cible_choc([Prep|X]) --> [statique([Prep|X])], {prep_sur(Prep)}.	% un objet statique peut tre la cible d'un choc
agent_choc([par|X]) --> [acteur([par|X])].				% un acteur peut tre l'agent d'un choc

vchoc(V) --> [verbe_choc(V)].
vdchoc(V) --> [verbe_double_choc(V)].
vdpronominalchoc(V, [X]) -->
	[verbe_double_choc(V)],
	{pronom_reflechi(X), member(X,V)}.
vpronominalchoc(V, [X]) -->
	[verbe_choc(V)],
	{pronom_reflechi(X), member(X,V)}.

choc_par_defaut([], [choc([je],[him],[heurter],0)]).
choc_par_defaut(SortieChocs, SortieChocs).

%trouve_events(Events, ListeEvents) --> mot

% trouve la liste des acteurs et la liste des chocs pour l'instant
trouve_tout(ReportText, SortieActeurs, SortieChocs, SortieEvent) :-
	parse_all_text(ReportText, ParsedText),
	trouve_tout(ParsedText, [], [], [], SortieActeurs, SortieChocsProvisoire, SortieEvent),
	choc_par_defaut(SortieChocsProvisoire, SortieChocs).

trouve_tout([L|Rest], ActeursPrecedents, ChocsPrecedents, EventsPrecedents, SortieActeurs, SortieChocs,SortieEvent) :-
	mots( _, Acteurs, ListeChoc, /*Liste*/Events, L, []),
	!,
	trouve_choc(Choc, _, ListeChoc,[]),
	%trouve_events(Events, ListeEvents, []),
	append(ChocsPrecedents,Choc,NouveauxChocs),
	append(EventsPrecedents,Events,NouveauxEvents),
	append(ActeursPrecedents,Acteurs,NouveauxActeurs),
	trouve_tout(Rest, NouveauxActeurs, NouveauxChocs,NouveauxEvents, SortieActeurs, SortieChocs,SortieEvent).

trouve_tout([],SortieActeurs, SortieChocs, SortieEvent, SortieActeurs, SortieChocs, SortieEvent).








