% fiche.pl contains the predicates creating the fiche from the
% extracted information

%-----------------------------------------------------------------------------
% criture du formulaire
%  crire au format XML dans l'avenir


write_statics(StaticObjects) :-	
	nl,	
	write('STATIC ['),
	nl,
	write_static_objects(StaticObjects),
	write(']'),
	nl.
	
write_static_objects([static(Type, Properties) | StaticObjects]) :-
	write_object(Type, Properties),
	write_static_objects(StaticObjects).
write_static_objects([]).

write_object('ROAD', [Kind]) :-
	format('   ROAD [~n', []),
	format('      KIND = ~a ;~n', Kind),
	format('   ]~n', []).
write_object('TREE', [Id, X, Y]) :-
	format('   TREE [~n', []),
	format('      ID = ~a ;~n', Id),
	format('      COORD = (~d,~d) ;~n', [X, Y]),
	format('   ]~n', []).
write_object('STOPSIGN', [ X, Y]) :-
	format('   STOPSIGN [~n', []),
	format('      COORD = (~d,~d) ;~n', [X, Y]),
	format('   ]~n', []).
write_object('CROSSING', [ X, Y]) :-
	format('   CROSSING [~n', []),
	format('      COORD = (~d,~d) ;~n', [X, Y]),
	format('   ]~n', []).
write_object('LIGHT', [Id, Color, X, Y]) :-
	format('   LIGHT [~n', []),
	format('      ID = ~a ;~n', Id),
	format('      COLOR = ~a ;~n', Color),
	format('      COORD = (~d,~d) ;~n', [X, Y]),
	format('   ]~n', []).

write_dynamics(DynamicObjects) :-
	nl,
	write('DYNAMIC ['), 
	nl,
	write_dynamic_objects(DynamicObjects),
	write(']'),
	nl.
	
write_dynamic_objects(DynamicObjects) :-
	member(vehicle(X, Y, Z, T, _), DynamicObjects),
	subtract(DynamicObjects, [vehicle(X, Y, Z, T, _)], NewDynamicObjects),
	format('   VEHICLE [~n', []),
	format('      ID = ~a; ~n', X),
	format('      KIND = ~a; ~n', Y),
	format('      INITDIRECTION = ~a; ~n', Z),
	format('      CHAIN [~n', []),
	write_events(T),
	format('      ] ~n', []),
	format('   ] ~n', []),
	write_dynamic_objects(NewDynamicObjects).
write_dynamic_objects(_).

write_events([]).
% il faudrait l'integrer lorsque les vnements sont multiples
/*write_events([X]) :-
	format('         CRITICAL_EVENT [~n', []),
	format('            KIND = ~a; ~n', X),
	format('         ] ~n', []),
	write_events([]).
*/
write_events([X | Rest]) :-
	format('         EVENT [~n', []),
	format('            KIND = ~a; ~n', X),
	format('         ] ~n', []),
	write_events(Rest).

write_accidents(Accidents) :-
	nl,
	write('// Collision Objects'),
	nl,
	write('ACCIDENT ['), 
	nl,
	write_collisions(Accidents),
	write(']'),
	!.		
	
write_collisions(Accidents) :-
	member(collision2(A, B, C, D, E, _), Accidents),
	subtract(Accidents, [collision2(A, B, C, D, E, _)], NewAccidents),
	format('   COLLISION [~n', []),
	format('      ACTOR = ~a,~a; ~n', [A, B]),
	(   
	    atom(C),
	    format('      VICTIM = ~a,~a; ~n', [C, D])
	    ; 
	    format('      VICTIM = ~p,~a; ~n', [C, D])
	),
	write_coords(E),
	format('   ] ~n', []),
	write_collisions(NewAccidents).
write_collisions(_).

write_coords([]).
write_coords([X, Y]):-
	format('      COORD = (~a,~a) ;~n', [X, Y]).





