%% Programme d'extraction d'informations pour les constats MAIF.
%% Programm par Vincent Legendre et Sylvain Dupuy
%% Modifi par Pierre Nugues
%% Modifi par Simon Le Gloannec, Pierre Aubeuf et Cdric Mtais


% on utilise l'analyseur de Pierre Nugues pour extraire la configuration de la route
%
% appel  des prdicats dynamique a liminer: vehicle, acteur, collision

% Static objects: static(+TYPE, +Features)
% choc(+Subject, +Object, +VerbeGroup, +Index)
% vehicle(enonciateur, car, east, [driving_forward], N))
% vehicle(Id, Type, InitDir, EventList, N))
% collision(enonciateur, front, defaut_B, front, [], N2)),
% collision(+VehicleA, +PartA, VehicleB, PartB, +Coord, +N2)),
% entre_parentheses( +/-GroupeDeMots, Acteur),
% acteur( +/-ListeMots), le mot le plus significatif du groupe de mot est place en tete de liste
% init_direction( +/-Acteur, +/- DirectionInitiale),
% nontrie (+/-Liste) des elements nontries.


% Teste si un verbe est  la voie passive
is_passive([]) :- fail.
is_passive([X | R]):-
	aux_conj_gerund_have(_, [X], []),
	isreallypassif(R).
is_passive([X | R]):-
	aux_conj_have(_, [X], []),
	isreallypassif(R).
is_passive([X | _]):-
        aux_conj_be(_, [X], []).
is_passive([X | _]):-
	aux_conj_gerund_be(_, [X], []).
is_passive([_ | R]):-
	is_passive(R).


isreallypassif([]) :- fail.
isreallypassif([X | _]):-
	aux_conj_gerund_be(_, [X], []).
isreallypassif([X | _]):-
	aux_conj_be(_, [X], []).
isreallypassif([_ | R]):-
	isreallypassif(R).

% Trouve le sujet d'un verbe ************************************************
%find_subject(+Parsed, +VerbeGroup, -Subject, -Rest),

find_subject(A, B, C, D) :-
	find_subject(A, B, C, D, _).

find_subject([vg(Vg) | R], Vg, Subject, R, Subject).

find_subject([ng(X) | R], Vg, Subject, Rest, _) :-
	test_valid_actor(X),
	find_subject(R, Vg, Subject, Rest, X).

find_subject([_ | R], Vg, Subject, Rest, Temp) :-
	find_subject(R, Vg, Subject, Rest, Temp).

test_valid_actor(X) :-
	actor1(Actor),
	contains_word(X, Actor).
test_valid_actor([X | _]) :-
	is_proper_noun(X).	
test_valid_actor([_ | R]) :-
	test_valid_actor(R).

test_valid_victim(X) :-
	victim1(Victim),
	contains_word(X, Victim).
test_valid_victim([X | _]) :-
	is_proper_noun(X).
test_valid_victim([_ | R]) :-
	test_valid_victim(R).

% Trouve le responsable d'un grondif ************************************************
%find_responsible(+Parsed, +GGroup, -Subject, -Rest),

find_responsible(A, B, C, D) :-
	find_responsible(A, B, C, D, _).

find_responsible([gg(Gg) | R], Gg, Subject, R, Subject).
find_responsible([ng(X) | R], Gg, Subject, Rest, _) :-
	test_valid_actor(X),
	find_responsible(R, Gg, Subject, Rest, X).
find_responsible([_ | R], Gg, Subject, Rest, Temp) :-
	find_responsible(R, Gg, Subject, Rest, Temp).

% Trouve le sujet d'une action considre comme un ng  ************************************************
%find_responsible(+Parsed, +NGroup, -Subject, -Rest),

find_noun_subject(A, B, C, D) :-
	find_noun_subject(A, B, C, D, _).

find_noun_subject([ng(Ng) | R], Ng, Subject, R, Subject).
find_noun_subject([ng(X) | R], Ng, Subject, Rest, _) :-
	test_valid_actor(X),
	find_noun_subject(R, Ng, Subject, Rest, X).
find_noun_subject([_ | R], Ng, Subject, Rest, Temp) :-
	find_noun_subject(R, Ng, Subject, Rest, Temp).

% Trouve le "sujet" d'un verbe infinitif************************************************
%find_subject(+Parsed, +VerbeGroup, -Subject, -Rest),

find_subject_infinitive_verb(A, B, C, D) :-
	find_subject_infinitive_verb(A, B, C, D, _).

find_subject_infinitive_verb([infg(Vg) | R], Vg, Subject, R, Subject).

find_subject_infinitive_verb([ng(X) | R], Vg, Subject, Rest, _) :-
	test_valid_actor(X),
	find_subject_infinitive_verb(R, Vg, Subject, Rest, X).

find_subject_infinitive_verb([_ | R], Vg, Subject, Rest, Temp) :-
	find_subject_infinitive_verb(R, Vg, Subject, Rest, Temp).

% Trouve l'objet d'un verbe actif *******************************************

find_valid_object([ng(X) | R], _, X, R) :-
	test_valid_victim(X).
find_valid_object([_ | Rest], Vg, Obj, R) :-
	find_valid_object(Rest, Vg, Obj, R).

find_pronoun_object(R, Vg, [X], R):-
	contains_word2(Vg, [m, nous, l, me, le, la, lui], X).

find_default_object([ng(X)|R],_,X,R).
find_default_object([_ | Rest],Vg,Obj,R) :-
	find_default_object(Rest,Vg,Obj,R).

find_object(Parsed,Vg,Obj,R):-
	find_valid_object(Parsed,Vg,Obj,R).
find_object(Parsed,Vg,Obj,R):-
	find_pronoun_object(Parsed,Vg,Obj,R).
find_object(Parsed,Vg,Obj,R):-
	find_default_object(Parsed,Vg,Obj,R).

% Trouve la cible lors d'un double choc*************************************
find_valid_target([pg([Prep | X])| R], _, [Prep | X], R) :-
	Prep \= par,
	test_valid_victim(X).
find_valid_target([_ | Rest], Vg, Tg, R) :-
	find_valid_target(Rest, Vg, Tg, R).



find_default_target([pg([Prep | X])|R],_,[Prep | X],R) :-
	Prep \= par.
find_default_target([_ | Rest],Vg,Tg,R) :-
	find_default_target(Rest,Vg,Tg,R).

find_target(Parsed,Vg,Tg,R):-
	find_valid_target(Parsed,Vg,Tg,R).

find_target(Parsed,Vg,Tg,R):-
	find_default_object(Parsed,Vg,Tg,R).


% Trouve l'agent d'un verbe passif (introduit par 'par') ********************
find_agent([pg([par|X])|R],[par| X],R):-
	test_valid_actor(X).
find_agent([_ | Rest],Obj,R):-
	find_agent(Rest,Obj,R).

/* Ancienne version n'utilisant pas la DCG
% Ajout des collisions et des vhicules en fonction des chocs trouvs
find_collisions :-
	choc(Subject, Object, _, N),
	find_part(Subject, SubjectPart),
	find_part(Object, ObjectPart),
	check_static_victim(Object, Object2),
	test_enonciateur(Subject, ResSubject),
	test_enonciateur(Object2, ResObject),
	add_collision(ResSubject, SubjectPart, ResObject, ObjectPart, [], N),!,
	retract(choc(Subject, Object, _, N)),
	find_collisions.
find_collisions.
*/

%-----------------------------------------------------------------------------
% Ajout des collisions et des vhicules en fonction des chocs trouvs
find_collisions(ReportText, StaticObjects, NewStaticObjects, NonSorted, InfoActors, NewInfoActors, Collisions) :-
	write('Recherche des collisions...'), nl,
	trouve_tout(ReportText, _, SortieChocs, _),
	find_collisions_aux(SortieChocs, StaticObjects, NewStaticObjects, NonSorted, InfoActors, NewInfoActors, [], Collisions).

find_collisions_aux([choc(Subject, Object, _, N) | AutresChocs], StaticObjects, FinalStaticObjects, NonSorted, InfoActors, FinalInfoActors, Collisions, FinalCollisions) :-
	find_part(Subject, SubjectPart, InfoActors),
	find_part(Object, ObjectPart, InfoActors),
	check_static_victim(Object, Object2, StaticObjects, NewStaticObjects),
	add_collision(Subject, SubjectPart, Object2, ObjectPart, [], N, NewStaticObjects, NonSorted, InfoActors, NewInfoActors, Collisions, NewCollisions),
	!,
	find_collisions_aux(AutresChocs, NewStaticObjects, FinalStaticObjects, NonSorted, NewInfoActors, FinalInfoActors, NewCollisions, FinalCollisions).

find_collisions_aux([], StaticObjects, StaticObjects, _, InfoActors, InfoActors, Collisions, Collisions).

%-----------------------------------------------------------------------------
% insertion d'une collision impliquant deux acteurs connus
%add_collision(Subject, SubjectPart, Object, ObjectPart, [], N, StaticObjects, +NonSorted, InfoActors, NewInfoActors, Collisions, NewCollisions)

add_collision(Subject, SubjectPart, Object, ObjectPart, [], N, _, _, InfoActors, InfoActors, Collisions, NewCollisions) :-
	member(acteur([[X1] | T1]), InfoActors),
	member(acteur([[X2] | T2]), InfoActors),
	X2 \= X1,
	member(Object, [[X2] | T2]),
	member(Subject, [[X1] | T1]),
	\+ member(collision(X1, _, X2, _, _, _), Collisions),
	append(Collisions, [collision(X1, SubjectPart, X2, ObjectPart, [], N)], NewCollisions),
	write_ln('insertion d''une collision impliquant deux acteurs connus ').

% insertion d'une collision impliquant un acteur connu un acteur non reconnu
add_collision(Subject, SubjectPart, Object, ObjectPart, [], N, _, NonSorted, InfoActors, NewInfoActors, Collisions, NewCollisions) :-
	member(Subject, NonSorted),
	member(acteur([[X1] | T1]), InfoActors),
	member(acteur([[X2]|T2]), InfoActors),
	X2 \= X1,
	member(Object,[[X2] | T2]),
	member(Subject, NonSorted),
	\+ member(collision(X1, _, X2, _, _, _), Collisions),
	append(Collisions, [collision(X1, SubjectPart, X2, ObjectPart, [], N)], NewCollisions),
	append(T1, [Subject], NewT1),
	subtract(InfoActors, [acteur([[X1] | T1])], InfoActors2),
	append(InfoActors2, [acteur([[X1] | NewT1])], NewInfoActors),
	write_ln('insertion d''une collision un acteur connu un acteur non reconnu ').

% insertion d'une collision impliquant  un acteur connu et un statique
add_collision(Subject, SubjectPart, Object, ObjectPart, [], N, StaticObjects, _, InfoActors, InfoActors, Collisions, NewCollisions) :-	
	member(acteur([[X] | T]), InfoActors),
	member(Subject,[[X] | T]),
	member(static( _, [ Object, _, _]), StaticObjects),
	\+ member(collision(X, _, Object, _, _,_), Collisions),
	append(Collisions, [collision(X, SubjectPart, Object, ObjectPart, [], N)], NewCollisions).

% insertion d'une collision impliquant deux acteurs identiques => on garde le sujet, on change l'objet
add_collision(Subject, SubjectPart, _, ObjectPart, [], N, _, _, InfoActors, InfoActors, Collisions, NewCollisions) :-
	member(acteur([[X1] | T1]), InfoActors),
	member(acteur([[X2] | _]), InfoActors),
	X2 \= X1,
	member(Subject,[[X1] | T1]),
	\+ member(collision(X1, _, X2, _, _, _), Collisions),
	append(Collisions, [collision(X1, SubjectPart, X2, ObjectPart, [], N)], NewCollisions),
	write_ln('insertion d''une collision impliquant un acteur connu et un statique').


% insertion d'une collision impliquant d'un acteur non reconnu et un statique
add_collision(Subject, SubjectPart, Object, ObjectPart, [], N, _, NonSorted, InfoActors, NewInfoActors, Collisions, NewCollisions) :-
	member(Subject, NonSorted),
	% la plupart du temps, quand un objet statique est percut,
	% c'est l'enonciateur qui l'a fait
	X1 = enonciateur,
	member(acteur([[X1] | T1]), InfoActors),
	!,
	member(Subject, NonSorted),
	\+ member(collision(X1, _, Object, _, _, _), InfoActors),
	append(Collisions, [collision(X1, SubjectPart, Object, ObjectPart, [], N)], NewCollisions),
	append(T1, [Subject], NewT1),
	subtract(InfoActors, [acteur([[X1] | T1])], InfoActors2),
	append(InfoActors2, [acteur([[X1] | NewT1])], NewInfoActors),
	write_ln('insertion d''une collision impliquant un acteur non reconnu et un statique ').

add_collision(_, _, _, _, _, _, _, _, InfoActors, InfoActors, Collisions, Collisions).

% cherche une partie de vhicule dans une liste de mots
find_part(Liste, leftside, InfoActors) :-
	member(acteur(X), InfoActors),
	member(Liste, X),
	member(Y, X),
	contains_word(Y, [gauche]).
find_part(Liste, rightside, InfoActors) :-
	member(acteur(X), InfoActors),
	member(Liste, X),
	member(Y, X),
	contains_word(Y, [droit, droite]).
find_part(Liste, front, InfoActors):-
	member(acteur(X), InfoActors),
	member(Liste, X),
	member(Y, X),
	contains_word(Y, [avant, devant]).
find_part(Liste, rear, InfoActors) :-
	member(acteur(X), InfoActors),
	member(Liste, X),
	member(Y, X),
	contains_word(Y, [arrire, derrire]).
find_part(_ , unknown, _).

%-----------------------------------------------------------------------------
% Recherche s'il y a une victime statique et l'asserte ventuellement
check_static_victim(Object, Id, StaticObjects, StaticObjects) :-
	hyp(Arbre, arbre),
	member(Arbre, Object),
	member(static('TREE', [Id, _, _]), StaticObjects).
check_static_victim(Object, Id, StaticObjects, StaticObjects) :-
	hyp(RoadObstacle, road_obstacle),
	member(RoadObstacle, Object),
	member(static('TREE', [Id, _, _]), StaticObjects).
check_static_victim(Object, RoadObstacle, StaticObjects, NewStaticObjects) :-
	hyp(RoadObstacle, road_obstacle),
	member(RoadObstacle, Object),
	append(StaticObjects, [static('TREE', [RoadObstacle, -5, -5])], NewStaticObjects).
check_static_victim(Object, Object, StaticObjects, StaticObjects).

%-----------------------------------------------------------------------------
% Creation d'une collision par dfaut

defaut_collision :-
	\+ collision(_, _, _, _, _, _), % si il n'y a pas de collisions
	nl, write('Creation d une collision par dfaut !'), nl,
	acteur([[enonciateur] | _]),
	acteur([[Y] | _]),
	Y \= enonciateur,
	find_part([enonciateur], SubjectPart),
	find_part([Y], ObjectPart),
	assert(collision(enonciateur,SubjectPart,Y,ObjectPart,[0,0],1)).

defaut_collision.


%-----------------------------------------------------------------------------
% Affine les collisions en dterminant les coordonnes de certaines collisions

% cas ou trois voitures se suivent dont une est arrte 
% la derniere pousse la deuxieme sur la premiere

affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(acteur([[Acteur1] | _]), InfoActors),
	member(acteur([[Acteur2] | _]), InfoActors),
	member(acteur([[Acteur3] | _]), InfoActors),
	Acteur1 \== Acteur2,
	Acteur1 \== Acteur3,
	Acteur2 \== Acteur3,
	member(collision(Acteur1, X1, Acteur2, Y2, C1, N1), Collisions),
	member(collision(Acteur2, X2, Acteur3, Y3, C2, N2), Collisions),
	subtract(Collisions, [collision(Acteur1, X1, Acteur2, Y2, C1, N1), collision(Acteur2, X2, Acteur3, Y3, C2, N2)], Collisions2),
	append(Collisions2, [collision2(Acteur1, front, Acteur2, rear, ['-12', '1.5'], N1), collision2(Acteur2, front, Acteur3, rear, ['-6', '1.5'], N2)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas ou trois voitures se suivent dont une est arretee 
% la derniere pousse la deuxieme sur la premiere

affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(acteur([[Acteur1] | _]), InfoActors),
	member(acteur([[Acteur2] | _]), InfoActors),
	member(acteur([[Acteur3] | _]), InfoActors),
	Acteur1 \== Acteur2,
	Acteur2 \== Acteur3,
	Acteur1 \== Acteur3,
	member(collision(Acteur1, X1, Acteur2, Y2, C1, N1), Collisions),
	subtract(Collisions, [collision(Acteur1, X1, Acteur2, Y2, C1, N1)], Collisions2),
	append(Collisions2, [collision2(Acteur1, front, Acteur2, rear, ['-13', '1.5'], N1)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas ou deux voitures se suivent et la premiere est arretee
affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(collision(Actor, X, Victim, Y, C, N), Collisions),
	member(init_direction(Actor, east), InfoActors),
	member(init_direction(Victim, east), InfoActors),
	member(event(Victim, ListeEvent), InfoActors),
	member(stop, ListeEvent),
	subtract(Collisions, [collision(Actor, X, Victim, Y, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, front, Victim, rear, ['-7', '1.5'], N)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas ou deux voitures se suivent sans s'arrter 
% dont une tourne a droite
affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(collision(Actor, X, Victim, Y, C, N), Collisions),
	member(init_direction(Actor, east), InfoActors),
	member(init_direction(Victim, east), InfoActors),
	member(event(Victim, ListeEvent2), InfoActors),
	member(turn_right, ListeEvent2),
	subtract(Collisions, [collision(Actor, X, Victim, Y, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, X, Victim, Y, ['-3', '3'], N)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas ou deux voitures se suivent sans s arreter 
% dont une tourne a gauche
affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(collision(Actor, X, Victim, Y, C, N), Collisions),
	member(init_direction(Actor, east), InfoActors),
	member(init_direction(Victim, east), InfoActors),
	member(event(Victim, ListeEvent2), InfoActors),
	member(turn_left, ListeEvent2),
	subtract(Collisions, [collision(Actor, X, Victim, Y, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, X, Victim, Y, ['-3', '0'], N)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas ou deux voitures se suivent sans s arreter
affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(collision(Actor, unknown, Victim, unknown, C, N), Collisions),
	member(vehicle(Actor, _, east, ListeEvent1,_), InfoActors),
	member(vehicle(Victim, _, east, ListeEvent2,_), InfoActors),
	member(driving_forward, ListeEvent1),
	member(driving_forward, ListeEvent2),
	subtract(Collisions, [collision(Actor, unknown, Victim, unknown, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, front, Victim, rear, ['0', '1.5'], N)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas ou deux voitures se suivent sans s arreter
affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(collision(Actor, X, Victim, Y, C, N), Collisions),
	member(vehicle(Actor, _, east, ListeEvent1, _), InfoActors),
	member(vehicle(Victim, _, east, ListeEvent2, _), InfoActors),
	member(driving_forward, ListeEvent1),
	member(driving_forward, ListeEvent2),
	subtract(Collisions, [collision(Actor, X, Victim, Y, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, X, Victim, Y, ['0', '1.5'], N)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas d'un refus de priorit a droite
affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(collision(Actor, X, Victim, Y, C, N), Collisions),
	member(init_direction(_, east), InfoActors),
	member(init_direction(_, south), InfoActors),
	subtract(Collisions, [collision(Actor, X, Victim, Y, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, X, Victim, Y, ['-1.5', '1.5'], N)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).

% cas d'un refus de priorit a droite
affine_collision(InfoActors, Collisions, FinalCollisions) :-
	member(collision(Actor, X, Victim, Y, C, N), Collisions),
	member(init_direction(_, east), InfoActors),
	member(init_direction(_, north), InfoActors),
	subtract(Collisions, [collision(Actor, X, Victim, Y, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, X, Victim, Y, ['1.5', '1.5'], N)], NewCollisions),
	!,
	affine_collision(InfoActors, NewCollisions, FinalCollisions).


% cas ou on ne change rien.
affine_collision(_, Collisions, NewCollisions) :-
	member(collision(Actor, X, Victim, Y, C, N), Collisions),
	subtract(Collisions, [collision(Actor, X, Victim, Y, C, N)], Collisions2),
	append(Collisions2, [collision2(Actor, X, Victim, Y, C, N)], NewCollisions).

affine_collision(_, Collisions, Collisions).


%-----------------------------------------------------------------------------
%renvoie la liste de toutes les phrases decomposees
% -Bag
parse_all_text(ReportText, ParsedText) :-
	bagof(Parsed, Tokenized^parse(ReportText, Tokenized, Parsed), ParsedText).

%renvoie la liste de toutes les phrases tokenises
% -Bag
tokenize_all_text(ReportText, TokenizedText) :-
	bagof(Tokenized, Parsed^parse(ReportText, Tokenized, Parsed), TokenizedText).


% renvoie la liste de tous les noms reprsentant des acteurs 
% grce  la DCG
find_all_nouns(ReportText, SortieActeurs) :- 
	trouve_tout(ReportText, SortieActeurs, _, _).

%-----------------------------------------------------------------------------
% CRATION DE TOUS LES ACTEURS

% on trie tous les mots et on les met dans une liste
% un nom significatif : un vehicule ou un nom propre
% cas ou on russi  trouver des noms significatifs
cree_acteurs(ReportText, InfoActors, _, NonSorted) :-
	find_all_nouns(ReportText, NounGroups),
	% on ne garde que les acteurs avec un nom significatif
	setof([[X]], G^(member(G, NounGroups), is_defini(G, X)), R),
	describe_actors(enonciateur, 1, InfoActor1),
	describe_actors(R, 2, InfoActor2),
	append(InfoActor1, InfoActor2, FirstInfoActors),
	trie_acteurs(NounGroups, [enonciateur], FirstInfoActors, InfoActors, NonSorted),	
	nl,  write('L''algorithme de corfrence n''a pas tri : '), write(NonSorted), 
	nl.
% il faut ensuite affiner la recherche en affectant aux acteurs les lments non-tris
% => algorithme de corfrence

% cas ou on ne russi  trouver aucun nom significatif mais 
% on trouve un objet statique.
% l'acteur se cogne tout seul dans un objet statique.
cree_acteurs(ReportText, InfoActor1, StaticObjects, NonSorted) :-
	member(static('TREE', _), StaticObjects),
	find_all_nouns(ReportText, NounGroups),
%	road_obstacle(Obstacles),
%	contains_word2(ReportText, Obstacles, _),
	% creation de l'acteur 'enonciateur'
	describe_actors(enonciateur, 1, FirstInfoActor1),
	trie_acteurs(NounGroups, [enonciateur], FirstInfoActor1, InfoActor1, NonSorted),	
	nl,  write('L''algorithme de corfrence n''a pas tri :'), write(NonSorted),
	nl.

% cas ou on ne russi  trouver aucun nom significatif 
% cration d'un vhicule par dfaut
cree_acteurs(ReportText, InfoActors, _, NonSorted) :-
	find_all_nouns(ReportText, NounGroups),
	% creation de l'acteur 'enonciateur'
	describe_actors(enonciateur, 1, InfoActor1),
	% creation de l'autre acteur par dfaut
	InfoActor2 = [acteur([[vhicule], [il], [lui], [him]]), event(vhicule, []), init_direction(vhicule, east), acteur_numero(vhicule, 1)],
	!,
	append(InfoActor1, InfoActor2, FirstInfoActors),
	trie_acteurs(NounGroups, [enonciateur], FirstInfoActors, InfoActors, NonSorted),	
	nl, write('L''algorithme de corfrence n''a pas tri : '),
	write(NonSorted), nl.

% creation de l'acteur 'enonciateur'
describe_actors(enonciateur, N, InfoActor) :-
	InfoActor = [acteur([[enonciateur], [m], [me], [moi]]), event(enonciateur, []), init_direction(enonciateur, east), acteur_numero(enonciateur, N)].

% creation d'un autre acteur
describe_actors([[[Acteur]] | T], N, InfoActors) :-
	N1 is N + 1,
	describe_actors(T, N1, InfoOtherActors),
	append([acteur([[Acteur], [him]]), init_direction( Acteur, east), event(Acteur, []), acteur_numero(Acteur, N)], InfoOtherActors, InfoActors).

describe_actors([], _, []) :- !.

% on lit la liste des mots representant des acteurs potentiels
% trie_acteurs (+Listerepresentants,+ListeActeurs,+InfoActeur, +NouvInfoActeur, -Nontri)
% principe on lit la liste de representant, on change l'acteur 
% courant si on rencontre un nom significatif  
trie_acteurs([Mot | Reste], Acteurs, InfoActeur, InfoActeurFinal, NonTrie) :-
	once((
	is_me(Mot),
	member(acteur([[enonciateur] | Q]), InfoActeur),
	subtract(InfoActeur, [acteur([[enonciateur] | Q])], InfoActeur2),
	append(InfoActeur2, [acteur([[enonciateur], Mot | Q])], NouvInfoActeur)
       	)),
	trie_acteurs(Reste, Acteurs, NouvInfoActeur, InfoActeurFinal, NonTrie).
/*
% si on rencontre un mot significatif on le place comme acteur courant
% sauf s'il l'est deja
trie_acteurs([Mot|Reste], [ActeurCourant| AutresActeurs], NonTrie) :-
	once((
	acteur( [[ActeurCourant]| Q] ),
	is_defini(Mot,ActeurCourant),
	retract(acteur( [[ActeurCourant]| Q] )),
	assert(acteur( [[ActeurCourant], Mot | Q] )),
	)),
	trie_acteurs( Reste, [ActeurCourant| AutresActeurs] , NonTrie).

*/

% si on rencontre un mot significatif on le place comme acteur courant
trie_acteurs([Mot | Reste], Acteurs, InfoActeur, InfoActeurFinal, NonTrie) :-
	once((
	member(acteur([[NouvelActeur] | Q]), InfoActeur),
	is_defini(Mot, NouvelActeur),
	subtract(InfoActeur, [acteur([[NouvelActeur] | Q])], InfoActeur2),
	append(InfoActeur2, [acteur([[NouvelActeur], Mot | Q])], NouvInfoActeur)
	)),
	trie_acteurs( Reste, [NouvelActeur | Acteurs], NouvInfoActeur, InfoActeurFinal, NonTrie).

% si on rencontre un mot de conducteur on le met dans la liste
% de l'acteur courant (celui dont on a parle en dernier)
trie_acteurs([Mot | Reste], Acteurs, InfoActeur, InfoActeurFinal, NonTrie) :-
	once((
	member(acteur([[ActeurCourant] | Q]), InfoActeur),
	is_driver(Mot, ActeurCourant),
	ActeurCourant \== enonciateur,
	subtract(InfoActeur, [acteur([[ActeurCourant] | Q])], InfoActeur2),
	append(InfoActeur2, [acteur([[ActeurCourant], Mot | Q])], NouvInfoActeur)
	)),
	!,
	trie_acteurs(Reste, [ActeurCourant| Acteurs], NouvInfoActeur, InfoActeurFinal, NonTrie).

% si on rencontre un mot a la troisieme personne on le met dans la liste
% de l'acteur courant (celui dont on a parle en dernier)
trie_acteurs([Mot | Reste], [Acteur | Acteurs], InfoActeur, InfoActeurFinal, NonTrie) :-
	once((
	is_him(Mot),
	member(acteur([[Acteur] | Q]), InfoActeur),
	Acteur \== enonciateur,
	subtract(InfoActeur, [acteur([[Acteur] | Q])], InfoActeur2),
	append(InfoActeur2, [acteur([[Acteur], Mot | Q])], NouvInfoActeur)
	)),
	trie_acteurs( Reste, [Acteur | Acteurs], NouvInfoActeur, InfoActeurFinal, NonTrie).

% si on rencontre un vhicule indefini on le met dans la liste
% de l'acteur courant (celui dont on a parle en dernier) si celui ci
% est est vhicule cr par dfaut
trie_acteurs([Mot | Reste], [ActeurCourant | AutresActeurs], InfoActeur, InfoActeurFinal, NonTrie) :-
	once((
	is_vehicule(Mot),
	member(acteur([[NouvelActeur] | Q]), InfoActeur),
	indefini(NouvelActeur),
	NouvelActeur \== enonciateur,
	subtract(InfoActeur, [acteur([[NouvelActeur] | Q])], InfoActeur2),
	append(InfoActeur2, [acteur([[NouvelActeur], Mot | Q])], NouvInfoActeur)
	)),
	trie_acteurs( Reste, [NouvelActeur, ActeurCourant| AutresActeurs], NouvInfoActeur, InfoActeurFinal, NonTrie).

% si on rencontre un nous on le met dans deux listes : celle de l'enonciateur
% et celle de l'acteur dont on a parle en dernier
trie_acteurs([Mot | Reste], [Acteur | Acteurs], InfoActeur, InfoActeurFinal, NonTrie) :-
	once((
	is_us(Mot),
	member(acteur([[ActeurCourant] | Q1]), InfoActeur),
	member(acteur([[enonciateur] | Q2]), InfoActeur),
	ActeurCourant \== enonciateur,
	subtract(InfoActeur, [acteur([[ActeurCourant] | Q1]), acteur([[enonciateur] | Q2])], InfoActeur2),
	append(InfoActeur2, [acteur([[enonciateur], Mot | Q2]), acteur([[ActeurCourant], Mot | Q1])], NouvInfoActeur)
	)),
	trie_acteurs( Reste, [Acteur | Acteurs], NouvInfoActeur, InfoActeurFinal, NonTrie).

% on fait la meme chose dans le cas ou l'acteur courant est l'enonciateur
% on considere alors que le nouvel acteur courant est celui en deuxieme 
% place dans la liste des acteurs.
trie_acteurs([Mot | Reste], [enonciateur, ActeurCourant | AutresActeurs], InfoActeur, InfoActeurFinal, NonTrie) :-
	once((
	is_us(Mot),
	member(acteur([[ActeurCourant] | Q]), InfoActeur),
	ActeurCourant \== enonciateur,
	subtract(InfoActeur, [acteur([[ActeurCourant] | Q]), acteur([[enonciateur] | Q])], InfoActeur2),
	append(InfoActeur2, [acteur([[enonciateur], Mot | Q]), acteur([[ActeurCourant], Mot | Q])], NouvInfoActeur)
	)),
	trie_acteurs(Reste, [ActeurCourant | AutresActeurs], NouvInfoActeur, InfoActeurFinal, NonTrie).

% il reste a trouver une methode pour classer les indefins : la voiture, un vehicule ...

% si le mot n'a pas trouve sa place on le met dans la liste des mots non tries
trie_acteurs([Mot | Reste], Acteurs, InfoActeur, InfoActeurFinal, [Mot | NonTrie]) :-
	trie_acteurs(Reste, Acteurs, InfoActeur, InfoActeurFinal, NonTrie).

trie_acteurs([], _, InfoActeur, InfoActeur, []).

% cherche les vehicules precises entre parentheses
% pour dduire d'eventuelles corfrences

s(ReportText, Actors) :- parenthesized_actors(ReportText, Actors).

parenthesized_actors(ReportText, EntreParenthesesActors) :-
	parse_all_text(ReportText, Text),
	lit_entre_parentheses(Text, [], EntreParentheseActorsRev),
	reverse(EntreParentheseActorsRev, EntreParenthesesActors).

lit_entre_parentheses([Phrase | Queue], EntreParenthesesActors, FinalActors):-
	parenthese(Phrase, _, EntreParenthesesActors, NewEntreParentheses),
	lit_entre_parentheses(Queue, NewEntreParentheses, FinalActors).
lit_entre_parentheses([], ParenthesesActors, ParenthesesActors).

parenthese([], _, Actors, Actors).
parenthese(['(' | T], Rest, EntreParentheseActors, NewActors) :-
	lire_parenthese(T, Rest, EntreParentheseActors, NewActors).
parenthese([ng(X) | T], _, EntreParentheseActors, NewActors) :-
	is_actor(X),
	parenthese(T, X, EntreParentheseActors, NewActors).
parenthese([_ | T], Rest, EntreParentheseActors, NewActors):-
	parenthese(T, Rest, EntreParentheseActors, NewActors).	

lire_parenthese([')' | T], Rest, EntreParentheseActors, FinalActors):-
	parenthese(T, Rest, EntreParentheseActors, FinalActors).
% trouve le groupe nominal adjacent au groupe entre parenthese
lire_parenthese([ng(X) | T], Acteur,  EntreParentheseActors, FinalActors):-
	lire_parenthese(T, Acteur, [entre_parentheses(X, Acteur) | EntreParentheseActors], FinalActors).
lire_parenthese(_, _, Actors, Actors).

% association des mots entre parentheses et des acteurs

association_parentheses_acteurs(InfoActors, EntreParenthesesActors, NewInfoActors) :-
	% on choisit un mot entre parenthese et le mot le plus proche
	member(entre_parentheses(Mot, MotAdjacent), EntreParenthesesActors),
	member(acteur(Descriptif), InfoActors), % on choisit un acteur
	% on regarde si le mot adjacent represente cet acteur
	member(MotAdjacent, Descriptif),
	append(Descriptif, [Mot], NouveauDescriptif),
	subtract(InfoActors, [acteur(Descriptif)], InfoActors2),
	append(InfoActors2, [acteur(NouveauDescriptif)], NewInfoActors).
association_parentheses_acteurs(InfoActors, _, InfoActors).

% RECOUPEMENT DES ACTEURS REPRSENTANT LA MME PERSONNE
% explication il se peut que lors de la creation des acteurs l'un d'entre eux soit ddoubl
% (par exemple avec le groupe de mot mon vehicule A -> enonciateur et vehicule A)
% on va rassembler ces acteurs

associe_acteurs(InfoActors, NewInfoActors) :-
	member(acteur([[Acteur1] | Descriptif1]), InfoActors),
	member(acteur([[Acteur2] | Descriptif2]), InfoActors),
	Acteur1 \= Acteur2,
	member(MotCommun, Descriptif1),
	member(MotCommun, Descriptif2),
	is_defini(MotCommun, _),
	recolle(Acteur1, Acteur2, InfoActors, NewInfoActors),
	nl, write(Acteur1), write(' et '), write(Acteur2), write(' sont les mmes !'), 
	nl.
associe_acteurs(InfoActors, NewInfoActors) :-
	member(acteur([[Acteur1] | Descriptif1]), InfoActors),
	member(acteur([[Acteur2] | _]), InfoActors),
	Acteur1 \= Acteur2,
	member(Groupe, Descriptif1),
	is_defini(Groupe, Acteur2),
	recolle(Acteur1, Acteur2, InfoActors, NewInfoActors),
	nl, write(Acteur1), write(' et '), write(Acteur2), write(' sont les mmes !'), 
	nl.
associe_acteurs(InfoActors, InfoActors).

recolle(Acteur1, Acteur2, InfoActors, NewInfoActors) :-
	member(acteur([[Acteur1] | Descriptif1]), InfoActors),
	member(acteur([[Acteur2] | Descriptif2]), InfoActors),
	Acteur1 \= enonciateur ,
	append(Descriptif1, Descriptif2, DescriptifTotal),
	subtract(InfoActors, [acteur([[Acteur1] | Descriptif1]), init_direction(Acteur1, _), acteur([[Acteur2] | Descriptif2])], InfoActors2),
	append(InfoActors2, [acteur([[Acteur2] | DescriptifTotal])], NewInfoActors).

recolle(Acteur1, Acteur2, InfoActors, NewInfoActors) :-
	member(acteur([[Acteur1] | Descriptif1]), InfoActors),
	member(acteur([[Acteur2] | Descriptif2]), InfoActors),
	Acteur2 \= enonciateur ,
	append(Descriptif1, Descriptif2, DescriptifTotal),
	subtract(InfoActors, [acteur([[Acteur1] | Descriptif1]), init_direction(Acteur2, _), acteur([[Acteur2] | Descriptif2])], InfoActors2),
	append(InfoActors2, [acteur([[Acteur1]|DescriptifTotal])], NewInfoActors).

% si des mot de la liste non  trie on un rapport avec l'un des vhicules
trie_reste(NonSorted, InfoActors, EntreParenthesesActors, NewInfoActors) :-
	member(acteur(Acteur), InfoActors),
	member(entre_parentheses(Mot, MotAdjacent), EntreParenthesesActors),
	% on choisit un mot entre parenthese et le mot le plus proche
	member(Mot, Acteur),
	member(MotAdjacent, NonSorted),
	subtract(InfoActors, [acteur(Acteur)], InfoActors2),
	append(Acteur, [Mot], NouvelActeur),
	append(InfoActors2, [acteur(NouvelActeur)], NewInfoActors),
	nl.	
trie_reste(_, InfoActors, _, InfoActors).

affine_acteurs(NonSorted, InfoActors, EntreParenthesesActors, NewInfoActors) :-
	association_parentheses_acteurs(InfoActors, EntreParenthesesActors, ParenthesisInfoActors),
	associe_acteurs(ParenthesisInfoActors, AssocInfoActors),
	trie_reste(NonSorted, AssocInfoActors, EntreParenthesesActors, NewInfoActors).

% Si l'nonciateur n'apparait dans aucune collision 
% on le retire 

% il faut trouver un meilleur systme
affine_enonciateur(Collisions, InfoActors, NewInfoActors) :- 
	\+ existe_enonciateur(Collisions),
	subtract(InfoActors, [vehicle(enonciateur, _, _, _, _), acteur([[enonciateur]| _ ])], NewInfoActors),
	write_ln('Pas d''enonciateur.').

affine_enonciateur(_, InfoActors, InfoActors).

existe_enonciateur(Collisions) :-
	member(collision(enonciateur, _, _, _, _, _), Collisions).
existe_enonciateur(Collisions) :-
	member(collision(_, _,enonciateur,  _, _, _), Collisions).


% teste si un mot designe un acteur valable.
is_actor(X) :- 
	member(Y,X),
	actor(Y).
% teste si un mot est un nom propre
is_proper_noun(X) :-
	sub_atom(X, 0, 3, _, 'M._').
is_proper_noun(X) :-
	sub_atom(X, 0, 4, _, 'Mme_').
is_proper_noun(X) :-
	sub_atom(X, 0, 5, _, 'Mlle_').

% dit si X est un enonciateur valable
is_me(X) :-
	prem_pers(Y),
	member(Y,X).

% teste si un mot correspond  la premire personne
is_me(X) :-
	pronom_possessif_prem_pers(Y),
	vehicule(Z),
	member(Y,X),
	member(Z,X).

% teste si un mot correspond  la troisime personne
is_him(X) :-
	trois_pers(Y),
	member(Y,X).

is_him(X) :-
	pronom_possessif_trois_pers(Y),
	vehicule(Z),
	member(Y,X),
	member(Z,X).

% teste si un mot correspond  un vhicule
is_vehicule(X) :-
	vehicule(Y),
	member(Y,X).

is_vehicle(T, N) :-
	member(N,T),
	vehicule(N).

% teste si un mot correspond  la premire personne
% du pluriel
is_us(X) :-
	prem_pers_plur(Y),
	member(Y,X).

% teste si un groupe de mots contient le conducteur d'un vehicule donn
is_driver(DriverGroup,Vehicle) :-
	member(Y,DriverGroup),
	drive(Y,Vehicle).

% teste si un groupe de mots contient le conducteur d'un vehicule donn
is_defini(T,N) :-
	member(N,T),
	significatif(N).


significatif(X) :- defini(X).
significatif(X) :- is_proper_noun(X).


% AJOUT DES VNEMENTS 

trouve_events(ReportText, InfoActors, NewInfoActors) :-
	nl, write('Recherche des vnements... '), nl, 
	parse_all_text(ReportText, Parsed),
	trouve_events_aux(Parsed, InfoActors, NewInfoActors).

trouve_events_aux([Parsed | Rest], InfoActors, FinalInfoActors) :-
	cherche_events(Parsed, InfoActors, NewInfoActors),
	trouve_events_aux(Rest, NewInfoActors, FinalInfoActors).
trouve_events_aux([_ | Reste], InfoActors, NewInfoActors) :-
	trouve_events_aux(Reste, InfoActors, NewInfoActors).
trouve_events_aux([], InfoActors, InfoActors).

% vnements
cherche_events(Parsed, InfoActors, FinalInfoActors) :-
	% on teste la structure de la phrase
	member(vg(VerbeGroup), Parsed),
	find_subject(Parsed, VerbeGroup, Vehicule, Rest),
	confirm_dir(Rest, GroupDirection, Rest2),
	member(Verbe, VerbeGroup),
	member(Dir, GroupDirection),
	verb_mouv(Verbe, Dir, Event),
	% on cherche des acteurs valables
	member(acteur([[Acteur] | Descriptif]), InfoActors),
	member(Vehicule, Descriptif),
	% on modifie la liste d vnements
	ajoute_event(Acteur, Event, InfoActors, NewInfoActors),
	!,
	cherche_events(Rest2, NewInfoActors, FinalInfoActors).

cherche_events(Parsed, InfoActors, FinalInfoActors) :-
	% on teste la structure de la phrase
	member(vg(VerbeGroup), Parsed),
	find_subject(Parsed, VerbeGroup, Vehicule, Rest),
	member(Verbe, VerbeGroup),
	verb_mouv(Verbe, _, Event),
	% on cherche des acteurs valables
	member(acteur([[Acteur] | Descriptif]), InfoActors),
	member(Vehicule, Descriptif),
	% on modifie la liste d venements
	ajoute_event(Acteur, Event, InfoActors, NewInfoActors),
	!,
	cherche_events(Rest, NewInfoActors, FinalInfoActors).

% L'ajout de cette regle provoque une regression pour le B44
cherche_events(Parsed, InfoActors, FinalInfoActors) :-
	% on teste la structure de la phrase
	member(gg(VerbeGroup), Parsed),
	find_responsible(Parsed, VerbeGroup, Vehicule, Rest),
	confirm_dir(Rest, GroupDirection, Rest2),
	member(Verbe, VerbeGroup),
	member(Dir, GroupDirection),
	verb_mouv(Verbe, Dir, Event),
	% on cherche des acteurs valables
	member(acteur([[Acteur] | Descriptif]), InfoActors),
	member(Vehicule, Descriptif),
	% on modifie la liste d venements
	ajoute_event(Acteur, Event, InfoActors, NewInfoActors),
	!,
	cherche_events(Rest2, NewInfoActors, FinalInfoActors).

cherche_events(Parsed, InfoActors, FinalInfoActors) :-
	% on teste la structure de la phrase
	member(gg(VerbeGroup), Parsed),
	find_responsible(Parsed, VerbeGroup, Vehicule, Rest),
	member(Verbe, VerbeGroup),
	verb_mouv(Verbe, _, Event),
	% on cherche des acteurs valables
	member(acteur([[Acteur] | Descriptif]), InfoActors),
	member(Vehicule, Descriptif),
	% on modifie la liste d venements
	ajoute_event(Acteur, Event, InfoActors, NewInfoActors),
	!,
	cherche_events(Rest, NewInfoActors, FinalInfoActors).

cherche_events(Parsed, InfoActors, FinalInfoActors) :-
	member(pg(PG), Parsed),
	a_arret(X),
	member(X, PG),
	member(acteur([[Acteur] | Descriptif]), InfoActors),
	member(Vehicule, Descriptif),
	member(ng(Vehicule), Parsed),
	ajoute_event(Acteur, [stop], InfoActors, NewInfoActors),
	!,
	delete(Parsed, pg(PG), Rest),
	cherche_events(Rest, NewInfoActors, FinalInfoActors).

cherche_events(Parsed, InfoActors, FinalInfoActors) :-
	% on teste la structure de la phrase
	member(infg(VerbeGroup), Parsed),
	find_subject_infinitive_verb(Parsed, VerbeGroup, Vehicule, Rest),
	confirm_dir(Rest, GroupDirection, Rest2),
	member(Verbe, VerbeGroup),
	member(Dir, GroupDirection),
	verb_mouv(Verbe, Dir, Event),
	% on cherche des acteurs valables
	member(acteur([[Acteur] | Descriptif]), InfoActors),
	member(Vehicule,Descriptif),
	% on modifie la liste d venements
	ajoute_event(Acteur, Event, InfoActors, NewInfoActors),
	!,
	cherche_events(Rest2, NewInfoActors, FinalInfoActors).

cherche_events(_, InfoActors, InfoActors).

% ajout des vnements
ajoute_event(Acteur, Event, InfoActors, NewInfoActors):-
	member(event(Acteur, ListEvent), InfoActors),
	last(X, ListEvent),
	[X] \= Event,
	subtract(InfoActors, [event(Acteur, ListEvent)], InfoActors2),
	append(ListEvent, Event, NouvelleListe),
	append(InfoActors2, [event(Acteur, NouvelleListe)], NewInfoActors).

ajoute_event(Acteur, Event, InfoActors, NewInfoActors):-
	member(event(Acteur,[]), InfoActors),
	subtract(InfoActors, [event(Acteur,[])], InfoActors2),
	append(InfoActors2, [event(Acteur, Event)], NewInfoActors).

% affine les vnements
affine_events. % reste  le faire ...


% il faudrait voir ou sont places les panneaux stop et arretes tous les vhicules qui proviennent de cet axe au dpart

% CRATION DES VHICULES 

cree_vehicules(InfoActors, NewInfoActors) :-
	write_ln('Cration des vhicules... '),
	setof( 
	       vehicle(X, Kind, Dir, ListEvent, N),
	       Y^Kind^Dir^ListEvent^N^(
	       member(acteur([[X] | Y]), InfoActors),
	       kind(X, Kind),
	       member(init_direction(X, Dir), InfoActors),
	       member(event(X, ListEvent), InfoActors),
	       member(acteur_numero(X, N), InfoActors)
	     ),
		L),
	append(L, InfoActors, NewInfoActors).

% si il y a un stop les voitures sont a l'arret par defaut
default_events(StaticObjects, InfoActors, FinalInfoActors) :-
	member(static('STOPSIGN', _), StaticObjects),
	member(vehicle(X, K, D, [], N), InfoActors),
	subtract(InfoActors, [vehicle(X, K, D, [], N)], InfoActors2),
	append(InfoActors2, [vehicle( X, K, D, [driving_forward, stop], N)], NewInfoActors),
	default_events(StaticObjects, NewInfoActors, FinalInfoActors).

% si il y a un feu rouge, les voitures sont  l'arrt par dfaut
default_events(StaticObjects, InfoActors, FinalInfoActors) :-
	member(static('LIGHT',['trafficlight', 'red', _ ]), StaticObjects),
	member(vehicle(X, K, D, [], N), InfoActors),
	subtract(InfoActors, [vehicle(X, K, D, [], N)], InfoActors2),
	append(InfoActors2, [vehicle( X, K, D, [driving_forward, stop], N)], NewInfoActors),
	default_events(StaticObjects, NewInfoActors, FinalInfoActors).


% si aucun evenement n'a ete affecte a un vehicule il va tout droit
default_events(StaticObjects, InfoActors, FinalInfoActors) :-
	member(vehicle(X, K, D, [], N), InfoActors),
	subtract(InfoActors, [vehicle(X, K, D, [], N)], InfoActors2),
	append(InfoActors2, [vehicle( X, K, D, [driving_forward], N)], NewInfoActors),
	default_events(StaticObjects, NewInfoActors, FinalInfoActors).

default_events(_, InfoActors, InfoActors).





















