% RECHERCHE DE LA DIRECTION INITIALE

% on suppose un repre o la voiture de l'enonciateur vient toujours de l'est
% la gauche sera le sud
% en face sera l'ouest
% la droite sera le nord

% on va d'abord chercher les voitures qui se suivent 
% et leur donner la meme direction initiale

% on cherche des mots comme devant ou derriere

% Le format des directions initiales est
% init_direction(Vehicule, Direction) par exemple init_direction(Actor, east)

cree_directions_initiales(ReportText, InfoActors, NewInfoActors) :-
	write_ln('Recherche des directions initiales... '),
	parse_all_text(ReportText, ParsedText), 
	cherche_direction_initiale(ParsedText, InfoActors, NewInfoActors),
	!.

cherche_direction_initiale([], InfoActors, InfoActors).
cherche_direction_initiale([Sentence | ParsedText], InfoActors, FinalInfoActors) :-
	find_direction_initiale(Sentence, InfoActors, NewInfoActors),
	cherche_direction_initiale(ParsedText, NewInfoActors, FinalInfoActors).
cherche_direction_initiale([_ | ParsedText], InfoActors, FinalInfoActors) :-
	cherche_direction_initiale(ParsedText, InfoActors, FinalInfoActors).

% recherche si un vehicule est place soit devant soit derrire un autre
find_direction_initiale(Parsed, InfoActors, NewInfoActors) :-
	member(pg([Prep | X]), Parsed),
	meme_axe(Prep),
	member(ng(Vehicule), Parsed),
	member(acteur([[Acteur1] | Descriptif1]), InfoActors),
	member(acteur([[Acteur2] | Descriptif2]), InfoActors),
	member(X, Descriptif2),
	member(Vehicule, Descriptif1),
	%on verifie que les mots ne designent pas le meme acteur
	\+ (Acteur2 == Acteur1),
	!,
	% on modifie les directions initiales
	nouvelle_direction(Acteur1, Acteur2, meme, InfoActors, NewInfoActors).

% recherche si un vehicule suit un autre
find_direction_initiale(Parsed, InfoActors, NewInfoActors) :-
	% on teste la structure de la phrase
	member(vg(VG), Parsed),
	member(Verbe, VG),
	verb_meme_axe(Verbe),
	member(Personne, VG),
	member(ng(Vehicule), Parsed),
	% on cherche des acteurs valables
	member(acteur([[Acteur1] | Descriptif1]), InfoActors),
	member(acteur([[Acteur2] | Descriptif2]), InfoActors),
	member([Personne], Descriptif1),
	member(Vehicule, Descriptif2),
	% on modifie les directions initiales
	nouvelle_direction(Acteur1, Acteur2, meme, InfoActors, NewInfoActors),
	!.
	
% Ici cette partie tait bugge. Remplacement avec une smantique
% approximative
%on efface le verbe et on en prend un autre
%	delete(Parsed, vg(VG), Parsed2), 
%	find_direction_initiale(Parsed2, NewInfoActors, FinalInfoActors).

% recherche si un vehicule vient d'un autre autre cote
find_direction_initiale(Parsed, InfoActors, NewInfoActors) :-
	% on teste la structure de la phrase
	member(ng(Vehicule), Parsed),
	member(vg(VG), Parsed),
	member(pg([Prep | GroupDirection]), Parsed),
	member(Verbe, VG),
	verb_prov(Verbe, Prep),
	% on cherche des acteurs valables
	member(acteur([[enonciateur] | _]), InfoActors),
	member(acteur([[Acteur2] | Descriptif2]), InfoActors),
	member(Vehicule, Descriptif2),
	\+ (Acteur2 == enonciateur),
	% on modifie les directions initiales
	dir_prov(GroupDirection, NouvelleDirection),
	nouvelle_direction(enonciateur, Acteur2, NouvelleDirection, InfoActors, NewInfoActors),
	!.

% Ici cette partie tait bugge. Remplacement avec une smantique
% approximative
	%on efface le verbe et on en prend un autre
%	delete(Parsed, vg(VG), Parsed2),  
%	find_direction_initiale(Parsed2, NewInfoActors, FinalInfoActors).

% recherche si un vehicule vient d'un autre autre cote
% avec un gerondif
find_direction_initiale(Parsed, InfoActors, NewInfoActors) :-
	% on teste la structure de la phrase
	member(ng(Vehicule), Parsed),
	member(gg(GG), Parsed),
	member(pg([Prep | GroupDirection]), Parsed),
	member(Verbe, GG),
	verb_prov(Verbe, Prep),
	% on cherche des acteurs valables
	member(acteur([[enonciateur] | _]), InfoActors),
	member(acteur([[Acteur2] | Descriptif2]), InfoActors),
	member(Vehicule, Descriptif2),
	\+ (Acteur2 == enonciateur),
	% on modifie les directions initiales
	dir_prov(GroupDirection, NouvelleDirection),
	nouvelle_direction(enonciateur, Acteur2, NouvelleDirection, InfoActors, NewInfoActors).

% Ici cette partie tait bugge. Remplacement avec une smantique
% approximative
	%on efface le verbe et on en prend un autre
%	delete(Parsed, gg(GG), Parsed2),  
%	find_direction_initiale(Parsed2, NewInfoActors, FinalInfoActors).

% cas d'un refus de priorit  droite
find_direction_initiale(Parsed, InfoActors, NewInfoActors) :-
	% on teste la structure de la phrase
	member(ng(Priorite), Parsed),
	member(priorit, Priorite),
	% on cherche des acteurs valables
	member(acteur([[enonciateur] | _]), InfoActors),
	member(acteur([[Acteur2] | _]), InfoActors),
	\+ (Acteur2 == enonciateur),
	nouvelle_direction(enonciateur, Acteur2, gauche, InfoActors, NewInfoActors).
%	find_direction_initiale(Parsed, NewInfoActors, FinalInfoActors).

find_direction_initiale(_, InfoActors, InfoActors).

% le repere se fait a partir de l'enonciateur
% on pourrait creer un repere base sur autre chose, il suffit de changer les predicats


% positionne deux acteurs sur le meme axe
nouvelle_direction(enonciateur, Acteur, meme, InfoActors, NewInfoActors) :-
	member(init_direction(enonciateur, X), InfoActors),
	member(init_direction(Acteur, _), InfoActors),
	subtract(InfoActors, [init_direction(Acteur, _)], InfoActors2),
	append(InfoActors2, [init_direction(Acteur, X)], NewInfoActors).

nouvelle_direction(Acteur, enonciateur, meme, InfoActors, NewInfoActors) :-
	member(init_direction(enonciateur, X), InfoActors),
	member(init_direction(Acteur, _), InfoActors),
	subtract(InfoActors, [init_direction(Acteur, _)], InfoActors2),
	append(InfoActors2, [init_direction(Acteur, X)], NewInfoActors).

% positionne un vehicule a gauche de l'enonciateur
nouvelle_direction(enonciateur, Acteur, gauche, InfoActors, NewInfoActors) :-
	member(init_direction(enonciateur, east), InfoActors),
	member(init_direction(Acteur, east), InfoActors),
	subtract(InfoActors, [init_direction(Acteur, east)], InfoActors2),
	append(InfoActors2, [init_direction(Acteur, south)], NewInfoActors).
	% on ne changera qu'une seule fois la direction initiale

% positionne un vehicule a droite de l'enonciateur
nouvelle_direction(enonciateur, Acteur, droite, InfoActors, NewInfoActors) :-
	member(init_direction(enonciateur, east), InfoActors),
	member(init_direction(Acteur, east), InfoActors),
	subtract(InfoActors, [init_direction(Acteur, east)], InfoActors2),
	append(InfoActors2, [init_direction(Acteur, north)], NewInfoActors).
	% on ne changera qu'une seule fois la direction initiale

% positionne un vehicule en face de l'enonciateur
nouvelle_direction(enonciateur, Acteur, face, InfoActors, NewInfoActors) :-
	member(init_direction(enonciateur, east), InfoActors),
	member(init_direction(Acteur, east), InfoActors),
	subtract(InfoActors, [init_direction(Acteur, east)], InfoActors2),
	append(InfoActors2, [init_direction(Acteur, west)], NewInfoActors).
	% on ne changera qu'une seule fois la direction initiale

% il reste a faire le test de "en face" pour les virages

% positionne l'enonciateur a gauche d un vehicule 
nouvelle_direction(Acteur, enonciateur, gauche, InfoActors, NewInfoActors) :-
	member(init_direction(Acteur, east), InfoActors),
	member(init_direction(enonciateur, east), InfoActors),
	subtract(InfoActors, [init_direction(enonciateur, east)], InfoActors2),
	append(InfoActors2, [init_direction(enonciateur, south)], NewInfoActors).

%Confirme la direction
confirm_dir([pg(X) | R], X, R).
confirm_dir([rg(X) | R], X, R).
confirm_dir([_ | Rest], Obj, R):-
	confirm_dir(Rest, Obj, R).

% il y a surement d'autres cas  traiter...

% reconcile_init_direction(+StaticObjects, +DirectionInfoActors, -NewDirectionInfoActors)
% reconcile some directions as cars should come from east and south
% in a turn_left

% left turn is east <--> south, so north  --> south
reconcile_init_directions(StaticObjects, DirectionInfoActors, NewDirectionInfoActors) :-
	member(static('ROAD', [turn_left]), StaticObjects),
	member(init_direction(_, east), DirectionInfoActors),
	member(init_direction(Vehicle2, north), DirectionInfoActors),
	subtract(DirectionInfoActors, [init_direction(Vehicle2, north)], DirectionInfoActors2),
	append(DirectionInfoActors2, [init_direction(Vehicle2, south)], NewDirectionInfoActors).

% left turn is east <--> south, so west  --> south
reconcile_init_directions(StaticObjects, DirectionInfoActors, NewDirectionInfoActors) :-
	member(static('ROAD', [turn_left]), StaticObjects),
	member(init_direction(_, east), DirectionInfoActors),
	member(init_direction(Vehicle2, west), DirectionInfoActors),
	subtract(DirectionInfoActors, [init_direction(Vehicle2, west)], DirectionInfoActors2),
	append(DirectionInfoActors2, [init_direction(Vehicle2, south)], NewDirectionInfoActors).

% right turn is east <--> south, so west  --> south

reconcile_init_directions(_, DirectionInfoActors, DirectionInfoActors).
