% The ontology.pl file contains the description of the world
% in the future use the same format as WordNet

% regroupement de tous les prdicats designants des acteurs
actor(X) :- is_proper_noun(X).
actor(X) :- vehicule(X).
actor(X) :- driver(X).
actor(X) :- pronom_personnel(X).

% teste si un mot est le conducteur d'un vehicule
drive(motard, moto).
drive(motocycliste, moto).
drive(conducteur, _).
drive(conductrice, _).
drive(cycliste, vlo).
drive(cycliste, bicyclette).
drive(automobiliste, automobile). %  complter.


% teste si un mot est un conducteur 
driver(motard).
driver(motocycliste).
driver(personne).
driver(conducteur).
driver(conductrice).
driver(cycliste).
driver(automobiliste). %  completer.

pronom_personnel(X) :- prem_pers(X).
pronom_personnel(X) :- trois_pers(X).
pronom_personnel(X) :- prem_pers_plur(X).

% mots a la premiere personne
prem_pers(je).
prem_pers(j).
prem_pers(moi).
prem_pers(me).
prem_pers(m).
prem_pers(X) :- prem_pers_plur(X).

prem_pers('Je').
prem_pers('J').
prem_pers('Moi').
prem_pers('Me').
prem_pers('M').

pronom_possessif_prem_pers(mon).
pronom_possessif_prem_pers(ma).
pronom_possessif_prem_pers(mes).
pronom_possessif_prem_pers('Mon').
pronom_possessif_prem_pers('Ma').
pronom_possessif_prem_pers('Mes').


% mots a la premire personne du pluriel
prem_pers_plur(nous).
prem_pers_plur(nos).
prem_pers_plur(notre).

prem_pers_plur('Nous').
prem_pers_plur('Nos').
prem_pers_plur('Notre').

% mots a la troisieme personne
trois_pers('Celui_ci'). % rajout de dernire minute
trois_pers(tiers). % rajout de dernire minute
%trois_pers(l). % rajout de dernire minute

trois_pers(il).
trois_pers(lui).
trois_pers(elle).
trois_pers('Il').
trois_pers('Elle').

pronom_possessif_trois_pers(son).
pronom_possessif_trois_pers(sa).
pronom_possessif_trois_pers(ses).
pronom_possessif_trois_pers('Sa').
pronom_possessif_trois_pers('Son').

pronom_reflechi(nous).
pronom_reflechi(me).
pronom_reflechi(m).
pronom_reflechi(s).
pronom_reflechi(se).
pronom_reflechi(l).


% articles dfinis
art_def(le).
art_def(la).
art_def(les).
art_def(l).
art_def('La').
art_def('Le').
art_def('Les').
art_def('L').

% ne pas confondre avec vehicle !
vehicule(X) :- indefini(X).
vehicule(X) :- defini(X).
vehicule(X) :- partie_vehicule(X).

% designe les parties d'un vehicule
partie_vehicule(aile).
partie_vehicule(phare).
partie_vehicule(ct).
partie_vehicule(remorque).
partie_vehicule(rtroviseur).
partie_vehicule(pare_chocs).

% liste de vehicules pas suffisamment explicites
indefini(voiture).
indefini(vhicule).
indefini(voitures).

% liste de vehicules suffisamment explicites
defini('B').
defini(vhicule_c).
defini(vhicule_b).
defini(vhicule_a).
defini(vhicule_A).
defini(vhicule_B).
%defini(voiture).
defini(camionnette).
defini(auto).
defini(moto).
defini(vlo).
defini(bicyclette).
defini(camion).
defini(semi_remorque).
defini('Renault').
defini(renault).
defini('R11').
defini('R21').
defini('4L').
defini('205').
defini('Ford').
defini('Clio').
defini(205).

% verbes de provenance avec leur prposition
verb_prov(venir, de).
verb_prov(venir, sur).
verb_prov(trouver, sur).
verb_prov(sortir, de).
verb_prov(arriver, de).
verb_prov(venir, en).
verb_prov(arriver, en).
verb_prov(arriver, sur).
verb_prov(surgir, de).
verb_prov(surgir, sur).
% Verbe voir assimil  un verbe de provenance.
verb_prov(voir, en).
verb_prov(voir, sur).

%verb_prov(descendre,_).
%verb_prov(rouler,dans).
%verb_prov(circuler,dans).

% verbe de destination avec leur prposition
verb_dest(aller, ).
verb_dest(rendre, ).
verb_dest(tourner, vers).
verb_dest(engager, dans).

% prepositions de destination
prep_dest([vers | _]).
prep_dest([en, direction, de | _]).


% prepositions de provenance
prep_prov([depuis | _]).


% changement de direction
verb_tourn(tourner, ).
verb_tourn(tourner, sur).
verb_tourn(tourner, vers).

% indique si un vehicule se trouve sur le meme axe
meme_axe('Devant').
meme_axe(devant).
meme_axe('Derrire').
meme_axe(derrire).

% verbe indiquant un positionnement sur le meme axe
verb_meme_axe(suivre).
verb_meme_axe(prcder).

% indique une direction de provenance
dir_prov(X,gauche) :- member(gauche,X).
dir_prov(X,face)   :- member(face,X).
dir_prov(X,face)   :- member(inverse,X).
dir_prov(X,droite) :- member(droite,X).
dir_prov(X,meme)   :- member(mme,X).

% indique de quel type est un vehicule
kind(_, car). % pour l'instant il n'y a que des voitures

% caraterisation des verbes de mouvement
verb_mouv(T, G , [turn_left]):-
	verb_tourner(T),
	a_gauche(G).
verb_mouv(T, D , [turn_right]):-
	verb_tourner(T),
	a_droite(D).
verb_mouv(A, rien, [driving_forward]) :-
	verb_avancer(A).
verb_mouv(S, rien, [stop]) :-
	verb_stop(S).
verb_mouv(E, A, [stop]) :-
	verb_etat(E),
	a_arret(A).
verb_mouv(D, rien, [overtake]) :-
	verb_doubler(D).


verb_tourner(tourner).
%verb_tourner(amorcer).
verb_tourner(virer).

verb_avancer(redmarrer).
verb_avancer(couper).
verb_avancer(dmarrer).
verb_avancer(continuer).
verb_avancer(partir).
verb_avancer(repartir).
verb_avancer(rouler).
verb_avancer(circuler).
verb_avancer(venir).
verb_avancer(arriver).
verb_avancer(rouler).
verb_avancer(engager).
verb_avancer(passer).
verb_avancer(franchir).

verb_stop(arrter).
verb_stop(stopper).
verb_stop(caler).
verb_stop(immobiliser).
verb_stop(stationner).

verb_doubler(doubler).
verb_doubler(dpasser).

verb_etat(tre).

a_arret(arrt).
a_arret(stationner).
a_arret(immobiliser).
a_arret(stationnement).

a_gauche(_gauche).
a_gauche(gauche).
a_droite(_droite).
a_droite(droite).

is_static(X) :- 
	member(Y, X),
	objet_statique(Y).

objet_statique(rambarde).
objet_statique(pilier).
objet_statique(rail).
objet_statique(clture).
objet_statique(panneau).
objet_statique(trottoir).
objet_statique(mur).
objet_statique(barrire).
objet_statique(arbre).
objet_statique(benne).
objet_statique(stop).

prep_sur(contre).
prep_sur(sur).
prep_sur(vers).

% base de connaissances cre par Sylvain Dupuy 

% feux tricolores
feu(['feu','feux']):- !.

vert(['vert','verts']):- !.

rouge(['rouge','rouges']):- !.

orange(['orange','oranges']):- !.

flashing(['clignotant','clignotants']):- !.

% Les personnes
actor1(['j', 'tiers', 205, 'nous', 'je', 'il', 'elle', 'automobiliste', 'vhicule', 'vhicule_a', 'vhicule_b', 'vhicule_c', 'cycliste', 'conducteur', 'voiture', 'moto', 'motocyclette', 'camion','B','P205','205','Ford','R21']) :- !.

victim1(['nous','arbre', 'rambarde', 'pilier', 'rail', 'clture', 'panneau', 'benne','trottoir', 'mur', 'automobiliste','barrire', 'vhicule', 'vhicule_a', 'vhicule_b', 'vhicule_c', 'cycliste', 'conducteur', 'voiture', 'moto', 'motocyclette', 'camion',205,'P205','205','B','Ford','R21']) :- !.

enonciateur(['je', 'j', 'mon', 'ma', 'me', 'm', 'vehicule_a','nous']) :- !.

% Les verbes de choc
verbe_choc(['heurter', 'taper', 'percuter', 'toucher', 'abmer', 'accrocher', 'frotter', 'cogner', 'emboutir', 'enfoncer']) :- !.

% Les verbes de double choc
verbe_double_choc(['pousser', 'projeter']) :- !.

%ngation
%negation(['ne', 'n', 'pas']) :- !.

% Les objets statiques.
%Les carrefours
hyp(carrefour, carrefour).
hyp(intersection, carrefour).
hyp(feu, carrefour).
hyp(feux, carrefour).
hyp(stop, carrefour).
hyp(clignotant, carrefour).
hyp(clignoter, carrefour).
hyp(cder, carrefour).
hyp(priorit, carrefour).
hyp(sortie, carrefour).

%Les virages
hyp(virage, virage).
hyp(courbe, virage).
hyp(tournant, virage).

%Les arbres
hyp(arbre, arbre).
hyp(pin, arbre).
hyp(sapin, arbre).
hyp(chne, arbre).

% Les stops
hyp(stop, stop).
hyp(cdez, stop). %discutable

% Les passages
hyp(clout, passage_cloute).

% Les obstacles
hyp(rambarde, road_obstacle).
hyp(pilier, road_obstacle).
hyp(rail, road_obstacle).
hyp(clture, road_obstacle).
hyp(panneau, road_obstacle).
hyp(trottoir, road_obstacle).
hyp(mur, road_obstacle).
hyp(barrire, road_obstacle).

%hyp(benne, road_obstacle). % Discutable. Il faut introduire une nouvelle catgorie d'objets mobiles en stationnement.

hyp(tourner, tourner).
hyp(virer, tourner).

hyp(sortir, sortir).
hyp(surgir, sortir).

























