% DTECTION DES OBJETS STATIQUES
%  rcrire en utilisant des DCG

% find_statics(+ReportText) extracts static objects
% from the report
find_statics(ReportText, StaticObjects) :-
	parse_all_text(ReportText, ParsedText),
	tokenize_all_text(ReportText, TokenizedText),
	find_road(TokenizedText, ParsedText, Roads),
	find_stopsign(TokenizedText, ParsedText, StopSign),
	find_trafficlight(TokenizedText, ParsedText, Lights), 
	find_crossing(TokenizedText, Crossing),
	find_arbre(TokenizedText, Trees),
	find_barriere(TokenizedText, Walls),
        flatten([Roads, StopSign, Lights, Crossing, Trees, Walls], StaticObjects),
	!.



% Extrait le type de route
find_road(TokenizedText, ParsedText, Roads) :-
	find_carrefour(TokenizedText, ParsedText, Roads).
find_road(TokenizedText, ParsedText, Roads) :-
	find_virage(TokenizedText, ParsedText, Roads).
% ligne droite par dfaut pour les types de routes
find_road(_, _, [static('ROAD', [straightroad])]) :-
	write('ligne droite'), nl.

% recherche par mot
find_carrefour(TokenizedText, _, [static('ROAD', [crossroad])]) :-
	member(Sentence, TokenizedText),
	hyp(Carrefour, carrefour),
	member(Carrefour, Sentence),
	write('carrefour'), nl.
%recherche par action
find_carrefour(TokenizedText, ParsedText, [static('ROAD', [crossroad])]) :-
	member(Sentence, ParsedSentence, TokenizedText, ParsedText),
	\+ member(virage, Sentence),
	hyp(Tourner, tourner),
	member(vg(V), ParsedSentence),
	member(Tourner, V),
	write('tourner, carrefour'), nl.
%recherche par action
find_carrefour(TokenizedText, ParsedText, [static('ROAD', [crossroad])]) :-
	member(Sentence, ParsedSentence, TokenizedText, ParsedText),
	\+ member(virage, Sentence),
	hyp(Tourner, tourner),
	member(infg(V), ParsedSentence),
	member(Tourner, V),
	write('tourner, carrefour'), nl.

%recherche par action
find_carrefour(TokenizedText, ParsedText, [static('ROAD', [crossroad])]) :-
	member(Sentence, ParsedSentence, TokenizedText, ParsedText),
	\+ member(virage, Sentence),
	hyp(Sortir, sortir),
	member(vg(V), ParsedSentence),
	member(Sortir, V),
	write('tourner, carrefour'), nl.
%recherche par action
find_carrefour(TokenizedText, ParsedText, [static('ROAD', [crossroad])]) :-
	member(Sentence, ParsedSentence, TokenizedText, ParsedText),
	\+ member(virage, Sentence),
	hyp(Sortir, sortir),
	member(infg(V), ParsedSentence),
	member(Sortir, V),
	write('tourner, carrefour'), nl.


find_virage(TokenizedText, ParsedText, [static('ROAD', [turn_left])]) :-
	contains_virage(TokenizedText),
	member(ParsedSentence, ParsedText),
	member(pg([_, gauche]), ParsedSentence),
	write('virage  gauche'), nl.
find_virage(TokenizedText, ParsedText, [static('ROAD', [turn_right])]) :-
	contains_virage(TokenizedText),
	member(ParsedSentence, ParsedText),
	member(pg([_, droite]), ParsedSentence),
	write('virage  droite'), nl.
% virage  gauche par dfaut pour les virages
find_virage(TokenizedText, _, [static('ROAD', [turn_left])]) :-
	contains_virage(TokenizedText),
	write('virage  gauche par dfaut'), nl.

contains_virage(TokenizedText) :-		% test si on a un virage
	member(Sentence, TokenizedText),
	hyp(Virage, virage),
	member(Virage, Sentence).

% Extrait les panneaux stop
find_stopsign(TokenizedText, _, [static('STOPSIGN', [-5, 5])]) :-
	member(Sentence, TokenizedText),
	hyp(Stop, stop),
	member(Stop, Sentence),
	write(stop), nl.
find_stopsign(_, _, []).

% Extrait les feux tricolores
find_trafficlight(TokenizedText, ParsedText, Lights) :-
	member(Sentence, TokenizedText),
	feu(Feu),
	contains_word(Sentence, Feu),
	find_redlight(TokenizedText, RedLight),
	find_greenlight(TokenizedText, GreenLight),
	find_orangelight(TokenizedText, ParsedText, AmberLight),
	find_inactivelight(TokenizedText, ParsedText, InactiveLight),
	append(RedLight, GreenLight, Light1),
	append(AmberLight, InactiveLight, Light2),
	append(Light1, Light2, Lights).
find_trafficlight(_, _, []).

% Extrait les feux rouges
find_redlight(TokenizedText, [static('LIGHT',[trafficlight, red, -5, 5])]) :-
	is_red(TokenizedText),
	\+ is_green(TokenizedText),
	write('feu rouge'), nl.
find_redlight(_, []).

% Extrait les feux verts
find_greenlight(TokenizedText, [static('LIGHT',[trafficlight, green, -5, 5])]) :-
	is_green(TokenizedText),
	\+ is_orange(TokenizedText),
	write('feu vert'), nl.
find_greenlight(_, []).

% Extrait les feux oranges
find_orangelight(TokenizedText, ParsedText, [static('LIGHT', [trafficlight, orange, -5, 5])]) :-
	is_orange(TokenizedText),
	\+ is_red(TokenizedText),
	\+ is_inactive(TokenizedText, ParsedText),
	write('feu orange'), nl.
find_orangelight(_, _, []).

% Extrait les feux tricolores ne fonctionnant pas : clignotants ou teints
find_inactivelight(TokenizedText, ParsedText, [static('LIGHT', [trafficlight, inactive, -5, 5])]) :-
	is_inactive(TokenizedText, ParsedText),
	write('feu hors-fonctionnement'),nl.
find_inactivelight(_, _, []).

% recherche des couleurs
is_red(TokenizedText) :-
	member(Sentence, TokenizedText),
	rouge(Rouge),
	contains_word(Sentence, Rouge).

is_green(TokenizedText) :-
	member(Sentence, TokenizedText),
	vert(Vert),
	contains_word(Sentence, Vert).

is_orange(TokenizedText) :-
	member(Sentence, TokenizedText),
	orange(Orange),
	contains_word(Sentence, Orange).

is_inactive(TokenizedText, _) :-
	member(Sentence, TokenizedText),
	flashing(Clignotant),
	contains_word(Sentence, Clignotant).
is_inactive(_, ParsedText) :-
	member(ParsedSentence, ParsedText),
	member(pg([hors, fonctionnement]), ParsedSentence).

% Extrait les passages clouts
find_crossing(TokenizedText, [static('CROSSING', [-5, 0])]) :-
	member(Sentence, TokenizedText),
	hyp(Crossing, passage_cloute),
	member(Crossing, Sentence),
	write('passage clout'),nl.
find_crossing(_, []).

% Extrait les arbres
find_arbre(TokenizedText, [static('TREE', [tree, -4, 4])]) :-
	member(Sentence, TokenizedText),
	hyp(Arbre, arbre),
	member(Arbre, Sentence),
	write(arbre), nl.
find_arbre(_, []).

find_barriere(TokenizedText, [static('TREE', [Wall, 4, 4])]) :-
	member(Sentence, TokenizedText),
	hyp(Wall, road_obstacle),
	member(Wall, Sentence),
	write(barrire), nl.
find_barriere(_, []).


% member/4 accesses the sentences and parsed sentence in the same order
member(FS, FPS, [FS | _], [FPS | _]).
member(FS, FPS, [_ | S], [_ | PS]) :-
	member(FS, FPS, S, PS).














